# Copyright (c) 2020 Stefan Neumann and Pauli Miettinen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import math
import numpy

import HelperFunctions

def estimatePQ(ps, qs, MG, cols):
	bestP = 1
	bestQ = 0
	bestCluster = set()

	minDiff = 2

	for p in ps:
		for q in qs:
			if q >= p:
				continue

			estP, estQ, cluster = estimatePQSingleCluster(p,q,MG,cols)

			diff = numpy.abs(p - estP) + numpy.abs(q - estQ)

			if diff < minDiff:
				minDiff = diff
				bestP = p
				bestQ = q
				bestCluster = cluster

	return bestP, bestQ, bestCluster

def estimatePQSingleCluster(p,q,MG,cols):
	threshold = roundingThreshold(p,q)
	cluster = set( HelperFunctions.recoverRightCluster(MG, threshold) )

	if len(cluster) == 0:
		return numpy.Inf, -numpy.Inf, []

	totalOnes = 0
	onesInCluster = 0
	for key in MG.counters:
		value = MG.counters[key]

		totalOnes += value
		if key in cluster:
			onesInCluster += value

	m = MG.numVertices
	n = cols
	k = len(cluster)

	areaCluster = m*k
	areaOutsideCluster = m*(n-k)
	onesOutsideCluster = totalOnes - onesInCluster

	estP = onesInCluster / areaCluster
	estQ = onesOutsideCluster / areaOutsideCluster

	return estP, estQ, cluster

def roundingThreshold(p,q):
	return math.log( (1-q)/(1-p), 2 ) / math.log(p*(1-q) / q / (1-p), 2);

